/*
 * task.c
 *
 * Copyright (c) 1995-1997, John Kilburg <john@cs.unlv.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "port_before.h"

#include <stdio.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "port_after.h"

#include "ChimeraP.h"

struct ChimeraTaskP
{
  ChimeraTaskProc func;
  void            *closure;
  bool            removed;
};

static Boolean ScheduleTaskProc _ArgProto((XtPointer));

/*
 * ScheduleTaskProc
 */
static Boolean
ScheduleTaskProc(closure)
XtPointer closure;
{
  ChimeraResources cres = (ChimeraResources)closure;
  ChimeraTask wt;

  while ((wt = (ChimeraTask)GListPop(cres->tasks)) != NULL)
  {
    if (!wt->removed) (wt->func)(wt->closure);
    GListAddTail(cres->oldtasks, wt);
  }

  cres->wpid = 0;

  return(True);
}

/*
 * TaskSchedule
 */
ChimeraTask
TaskSchedule(cres, func, closure)
ChimeraResources cres;
ChimeraTaskProc func;
void *closure;
{
  ChimeraTask wt;

  if ((wt = (ChimeraTask)GListPop(cres->oldtasks)) == NULL)
  {
    wt = (ChimeraTask)MPCGet(cres->mp, sizeof(struct ChimeraTaskP));
  }
  wt->func = func;
  wt->closure = closure;
  wt->removed = false;

  GListAddTail(cres->tasks, wt);

  if (cres->wpid == 0)
  {
    cres->wpid = XtAppAddWorkProc(cres->appcon, ScheduleTaskProc, cres);
    myassert(cres->wpid != 0, "Made bad assumption about XtAppAddWorkProc");
  }

  return(wt);
}

/*
 * TaskRemove
 */
void
TaskRemove(cres, wt)
ChimeraResources cres;
ChimeraTask wt;
{
  GList list;
  ChimeraTask c;

  list = cres->tasks;
  for (c = (ChimeraTask)GListGetHead(list); c != NULL;
       c = (ChimeraTask)GListGetNext(list))
  {
    if (c == wt)
    {
      c->removed = true;
      break;
    }
  }

  return;
}
